/*
    FEARLUS model0-6-2: CautiousImitativeOptimumStrategy.m
    Copyright (C) 1999-2002  Macaulay Institute
    Copyright (C) 1999-2002  Macaulay Institute
    Copyright (C) 1999-2002  Macaulay Institute

    This file is part of FEARLUS model0-6-2, an agent-based model of land use
    change.

    FEARLUS model0-6-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Implementation for the CautiousImitativeOptimumStrategy object. 

*/

#import "CautiousImitativeOptimumStrategy.h"
#import "Number.h"
#import "AssocArray.h"
#import "SelectUseBucket.h"
#import "LandUse.h"
#import "LandParcel.h"
#import "LandManager.h"
#import "Environment.h"
#import "Parameter.h"
#import "FearlusArguments.h"
#import "BitString.h"
#import "MiscFunc.h"

@implementation CautiousImitativeOptimumStrategy
/*

create:withManager:andParameters:

Create the strategy, setting the manager and parameters.

*/

+(id <ImitativeStrategy, HistoricalStrategy>)create: aZone
	   withManager: (LandManager *)lmgr
	 andParameters: (Parameter *)p {
  CautiousImitativeOptimumStrategy *obj;

  obj = [super create: aZone];
  obj->lm = lmgr;
  obj->parameter = p;
  return obj;
}

/*

decideLandUseForParcel:

Use a cautious imitative optimum strategy to choose the land use, making use
of variance information. Neighbourhood weighting is ignored, and so is
neighbourhood noise..

*/

-(LandUse *)decideLandUseForParcel: (LandParcel *)lp {
  SelectUseBucket *bucket;
  id parcelIndex, tempZone;
  LandParcel *myParcel, *neighbouringParcel;
  LandManager *neighbour;
  LandUse *lu;
  AssocArray *vararr;
  int i;
  unsigned yield;
  unsigned match;
  BitString *bs;
  double getExpectedYieldVar(unsigned yield, unsigned antimatch,
			     unsigned lubits, double *var) {
    double eyield, eyield2, total;
    int i;

    total = (double)[MiscFunc combineN: (int)lubits r: (int)yield];
    eyield = 0.0;
    eyield2 = 0.0;
    if([FearlusArguments debugInnerCoreMethods]) {
      printf("i: Calculation of Expected Yield:\n\tYield = %u, antimatch = %u,"
	     " land use bits = %u\n", yield, antimatch, lubits);
      printf("\tYield Yield^2        No. of ways             Out of   "
	     "Probability\n");
    }
    for(i = 0; i <= (int)yield; i++) {
      double xi = (double)((int)yield + (int)antimatch - (2 * i));
      double pi = (double)[MiscFunc combineN: (int)antimatch r: i]
	* (double)[MiscFunc combineN: (int)(lubits - antimatch)
			    r: (int)yield - i];
      eyield += xi * pi;
      eyield2 += xi * xi * pi;
      if([FearlusArguments debugInnerCoreMethods]) {
	printf("\t% 5g % 7g % 18g % 18g % 13g\n",
	       xi, xi * xi, pi, total, (total > 0.0 ? xi / total : 0.0));
      }
    }
    eyield /= total;
    eyield2 /= total;

    (*var) = eyield2 - (eyield * eyield);
    return eyield;
  }

  tempZone = [Zone create: scratchZone];
  bucket = [SelectUseBucket create: tempZone
			    withParameters: parameter
			    andLandUses: [[lp getEnvironment] getLandUses]];
  vararr = [AssocArray createBegin: tempZone];
  [vararr setSize: [parameter nUses]];
  vararr = [vararr createEnd];
  // Loop through land parcels owned by the land manager
  if([FearlusArguments debugCoreMethods]) {
    printf("S: %s: Using imitative optimum strategy of land manager %u to "
	   "determine land use for LP = land parcel %u at (%d, %d)\n",
	   sel_get_name(_cmd), [lm getPIN], [lp getPIN], [lp getX], [lp getY]);
  }
  
  for(parcelIndex = [[lm getLandParcels] begin: tempZone], [parcelIndex next];
      [parcelIndex getLoc] == Member;
      [parcelIndex next]) {
    double score, variance;
    Number *n;

    myParcel = [parcelIndex get];
    yield = [myParcel getLastYield];
    lu = [myParcel getLandUse];
    match = [[lp getAttributes] getMatchWith: [myParcel getAttributes]];
    score = getExpectedYieldVar(yield,
				[parameter parcelBitStringSize] - match,
				[parameter useBitStringSize], &variance);
    n = [vararr getObjectWithKey: lu];
    if(n == nil) {
      [bucket setScore: score inBucketForLandUse: lu];
      n = [Number create: tempZone];
      [n setDouble: variance];
      if(![vararr addObject: n withKey: lu]) {
	fprintf(stderr, "CautiousImitativeOptimumStrategy: Adding variance %g "
		"of land use %u to the associative array failed\n",
		variance, [lu getPIN]);
	abort();
      }
      if([FearlusArguments debugCoreMethods]) {
	printf("S: %s: Found new land use %u at land parcel %u at (%d, %d) "
	       " with antimatch to LP of %u and yield %u. Expected yield "
	       " (= score for LU) = %g. Variance = %g\n", sel_get_name(_cmd),
	       [lu getPIN], [myParcel getPIN], [myParcel getX],
	       [myParcel getY], [parameter parcelBitStringSize] - match, 
	       yield, score, variance);
      }
    }
    else if(variance < [n getDouble]) {
      [bucket setScore: score inBucketForLandUse: lu];
      [n setDouble: variance];
      if([FearlusArguments debugCoreMethods]) {
	printf("S: %s: Found expected yield (%g) for land use %u with lower "
	       "variance (%g) at land parcel %u at (%d, %d).\n",
	       sel_get_name(_cmd), score, [lu getPIN], variance,
	       [myParcel getPIN], [myParcel getX], [myParcel getY]);
      }
    }
    else {
      if([FearlusArguments debugCoreMethods]) {
	printf("S: %s: Ignoring expected yield (%g) for land use %u with "
	       "higher variance (%g) at land parcel %u at (%d, %d).\n",
	       sel_get_name(_cmd), score, [lu getPIN], variance,
	       [myParcel getPIN], [myParcel getX], [myParcel getY]);
      }
    }
  }
  [parcelIndex drop];
  // Loop through the neighbours, and through their land parcels
  bs = [BitString create: tempZone
		  setBitCount: [parameter parcelBitStringSize]];
  for(i = 0; (neighbour = [lm getNextNeighbour: i]); i |= 1) {
    if([FearlusArguments debugCoreMethods]) {
      printf("S: %s: Adding scores for land uses of land parcels belonging"
	     " to land manager %u\n", sel_get_name(_cmd), [neighbour getPIN]);
    }
    for(parcelIndex = [[neighbour getLandParcels] begin: tempZone],
	  [parcelIndex next];
	[parcelIndex getLoc] == Member;
	[parcelIndex next]) {
      double score, variance;
      Number *n;

      neighbouringParcel = [parcelIndex get];
      lu = [neighbouringParcel getLandUse];
      [bs setToCopyOf: [neighbouringParcel getAttributes]];
      [bs mutateWithProbability: [lm getNeighbourNoise]];
      match = [[lp getAttributes] getMatchWith: bs];
      yield = [neighbouringParcel getLastYield];
      score = getExpectedYieldVar(yield,
				  [parameter parcelBitStringSize] - match,
				  [parameter useBitStringSize], &variance);
      n = [vararr getObjectWithKey: lu];
      if(n == nil) {
	[bucket setScore: score inBucketForLandUse: lu];
	n = [Number create: tempZone];
	[n setDouble: variance];
	if(![vararr addObject: n withKey: lu]) {
	  fprintf(stderr, "CautiousImitativeOptimumStrategy: Adding variance"
		  " %g of land use %u to the associative array failed\n",
		  variance, [lu getPIN]);
	  abort();
	}

	if([FearlusArguments debugCoreMethods]) {
	  printf("S: %s: Found new land use %u at neighbour's land parcel %u "
		 "at (%d, %d) with antimatch to LP of %u and yield %u"
		 "Expected yield = %g. Variance = %g.\n",
		 sel_get_name(_cmd),
		 [lu getPIN], [neighbouringParcel getPIN],
		 [neighbouringParcel getX], [neighbouringParcel getY],
		 [parameter parcelBitStringSize] - match, yield, score,
		 variance);
	}
      }
      else if(variance < [n getDouble]) {
	[bucket setScore: score inBucketForLandUse: lu];
	[n setDouble: variance];
	if([FearlusArguments debugCoreMethods]) {
	  printf("S: %s: Found expected yield (%g) for land use %u with lower "
		 "variance (%g) at neighbouring land parcel %u at (%d, %d).\n",
		 sel_get_name(_cmd), score, [lu getPIN], variance,
		 [neighbouringParcel getPIN],
		 [neighbouringParcel getX], [neighbouringParcel getY]);
	}
      }
      else {
	if([FearlusArguments debugCoreMethods]) {
	  printf("S: %s: Ignoring expected yield (%g) for land use %u with "
		 "higher variance (%g) at neighbouring parcel %u at (%d, %d)."
		 "\n", sel_get_name(_cmd), score, [lu getPIN], variance,
		 [neighbouringParcel getPIN],
		 [neighbouringParcel getX], [neighbouringParcel getY]);
	}
      }
    }
    [parcelIndex drop];
  }
  lu = [bucket getRandomOptimumChoice];
  [bucket drop];
  [bs drop];
  [tempZone drop];
  return lu;
}

@end
